﻿using Serilog.Filters;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.PortableExecutable;
using System.Text;
using System.Threading.Tasks;

namespace AntiAircraftGun
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetAntiAircraftGunsGeneric<T>
        where T : class, IEquatable<T>
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetAntiAircraftGunsGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="antiAircraftGun">Добавляемое орудие</param>
        /// <returns></returns>
        public int Insert(T antiAircraftGun)
        {
            if (_places.Count < _maxCount) return Insert(antiAircraftGun, 0);
            else throw new StorageOverflowException(_maxCount);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="antiAircraftGun">Добавляемое орудие</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T antiAircraftGun, int position)
        {
            if (_places.All(p => p.Equals(antiAircraftGun) == false))
            {
                if (position >= _maxCount) throw new StorageOverflowException(_maxCount);
                _places.Insert(position, antiAircraftGun);
                return position;
            }
            return - 1;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position >= _places.Count) throw new AntiAircraftGunNotFoundException(position);
            if (_places[position] == null) throw new AntiAircraftGunNotFoundException(position);
            T removed = _places[position];
            _places.RemoveAt(position);
            return removed;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                if (position < 0 || position >= _places.Count) return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position >= _places.Count) return;
                _places.Insert(position, value);
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetAntiAircraftGuns()
        {
            foreach (var antiAircraftGun in _places)
            {
                if (antiAircraftGun != null)
                {
                    yield return antiAircraftGun;
                }
                else
                {
                    yield break;
                }
            }
        }
        /// <summary>
        /// Сортировка набора объектов
        /// </summary>
        /// <param name="comparer"></param>
        public void SortSet(IComparer<T> comparer)
        {
            if (comparer == null)
            {
                return;
            }
            _places.Sort(comparer);
        }
    }
}
